@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 
 #include <memory>
 
-#include "base/callback.h"
+#include "base/functional/callback.h"
 #include "cc/cc_export.h"
 
 namespace base {
@@ -16,16 +16,15 @@ class Value;
 }  // namespace base
 
 namespace cc {
-
-class Layer;
 class LayerTreeHost;
 class PictureLayer;
 class MicroBenchmarkImpl;
+
 class CC_EXPORT MicroBenchmark {
  public:
-  typedef base::Callback<void(std::unique_ptr<base::Value>)> DoneCallback;
+  using DoneCallback = base::OnceCallback<void(base::Value)>;
 
-  explicit MicroBenchmark(const DoneCallback& callback);
+  explicit MicroBenchmark(DoneCallback callback);
   virtual ~MicroBenchmark();
 
   bool IsDone() const;
@@ -33,17 +32,16 @@ class CC_EXPORT MicroBenchmark {
   int id() const { return id_; }
   void set_id(int id) { id_ = id; }
 
-  virtual void RunOnLayer(Layer* layer);
   virtual void RunOnLayer(PictureLayer* layer);
 
-  virtual bool ProcessMessage(std::unique_ptr<base::Value> value);
+  virtual bool ProcessMessage(base::Value message);
 
   bool ProcessedForBenchmarkImpl() const;
   std::unique_ptr<MicroBenchmarkImpl> GetBenchmarkImpl(
       scoped_refptr<base::SingleThreadTaskRunner> origin_task_runner);
 
  protected:
-  void NotifyDone(std::unique_ptr<base::Value> result);
+  void NotifyDone(base::Value result);
 
   virtual std::unique_ptr<MicroBenchmarkImpl> CreateBenchmarkImpl(
       scoped_refptr<base::SingleThreadTaskRunner> origin_task_runner);
