@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,16 +7,16 @@
 #include <memory>
 #include <string>
 
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
+#include "base/memory/raw_ptr.h"
 #include "base/values.h"
-#include "chrome/browser/extensions/api/content_settings/content_settings_service.h"
 #include "chrome/browser/extensions/api/preference/preference_api.h"
 #include "chrome/browser/extensions/extension_prefs_unittest.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "components/prefs/mock_pref_change_callback.h"
+#include "extensions/browser/api/content_settings/content_settings_service.h"
 #include "extensions/browser/extension_prefs.h"
+#include "extensions/browser/extension_prefs_helper.h"
 #include "extensions/common/extension.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -39,35 +39,6 @@ const char kDefaultPref4[] = "default pref 4";
 
 }  // namespace
 
-// An implementation of the PreferenceAPI which returns the ExtensionPrefs and
-// ExtensionPrefValueMap from the TestExtensionPrefs, rather than from a
-// profile (which we don't create in unittests).
-class TestPreferenceAPI : public PreferenceAPIBase {
- public:
-  explicit TestPreferenceAPI(TestExtensionPrefs* test_extension_prefs,
-                             ContentSettingsService* content_settings)
-      : test_extension_prefs_(test_extension_prefs),
-        content_settings_(content_settings) {}
-  ~TestPreferenceAPI() {}
-
- private:
-  // PreferenceAPIBase implementation.
-  ExtensionPrefs* extension_prefs() override {
-    return test_extension_prefs_->prefs();
-  }
-  ExtensionPrefValueMap* extension_pref_value_map() override {
-    return test_extension_prefs_->extension_pref_value_map();
-  }
-  scoped_refptr<ContentSettingsStore> content_settings_store() override {
-    return content_settings_->content_settings_store();
-  }
-
-  TestExtensionPrefs* test_extension_prefs_;
-  ContentSettingsService* content_settings_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestPreferenceAPI);
-};
-
 class ExtensionControlledPrefsTest : public PrefsPrepopulatedTestBase {
  public:
   ExtensionControlledPrefsTest();
@@ -76,14 +47,14 @@ class ExtensionControlledPrefsTest : public PrefsPrepopulatedTestBase {
   void RegisterPreferences(user_prefs::PrefRegistrySyncable* registry) override;
   void InstallExtensionControlledPref(Extension* extension,
                                       const std::string& key,
-                                      base::Value* value);
+                                      base::Value value);
   void InstallExtensionControlledPrefIncognito(Extension* extension,
                                                const std::string& key,
-                                               base::Value* value);
+                                               base::Value value);
   void InstallExtensionControlledPrefIncognitoSessionOnly(
       Extension* extension,
       const std::string& key,
-      base::Value* value);
+      base::Value value);
   void InstallExtension(Extension* extension);
   void UninstallExtension(const std::string& extension_id);
 
@@ -96,15 +67,15 @@ class ExtensionControlledPrefsTest : public PrefsPrepopulatedTestBase {
   void EnsureExtensionUninstalled(const std::string& extension_id);
 
   TestingProfile profile_;
-  ContentSettingsService* content_settings_;
-  TestPreferenceAPI test_preference_api_;
+  raw_ptr<ContentSettingsService> content_settings_;
+  ExtensionPrefsHelper prefs_helper_;
 };
 
 ExtensionControlledPrefsTest::ExtensionControlledPrefsTest()
     : PrefsPrepopulatedTestBase(),
       content_settings_(ContentSettingsService::Get(&profile_)),
-      test_preference_api_(&prefs_, content_settings_) {
-  prefs_.prefs()->AddObserver(content_settings_);
+      prefs_helper_(prefs_.prefs(), prefs_.extension_pref_value_map()) {
+  content_settings_->OnExtensionPrefsAvailable(prefs_.prefs());
 }
 
 ExtensionControlledPrefsTest::~ExtensionControlledPrefsTest() {
@@ -121,28 +92,30 @@ void ExtensionControlledPrefsTest::RegisterPreferences(
 void ExtensionControlledPrefsTest::InstallExtensionControlledPref(
     Extension* extension,
     const std::string& key,
-    base::Value* value) {
+    base::Value value) {
   EnsureExtensionInstalled(extension);
-  test_preference_api_.SetExtensionControlledPref(
-      extension->id(), key, kExtensionPrefsScopeRegular, value);
+  prefs_helper_.SetExtensionControlledPref(
+      extension->id(), key, kExtensionPrefsScopeRegular, std::move(value));
 }
 
 void ExtensionControlledPrefsTest::InstallExtensionControlledPrefIncognito(
     Extension* extension,
     const std::string& key,
-    base::Value* value) {
+    base::Value value) {
   EnsureExtensionInstalled(extension);
-  test_preference_api_.SetExtensionControlledPref(
-      extension->id(), key, kExtensionPrefsScopeIncognitoPersistent, value);
+  prefs_helper_.SetExtensionControlledPref(
+      extension->id(), key, kExtensionPrefsScopeIncognitoPersistent,
+      std::move(value));
 }
 
 void ExtensionControlledPrefsTest::
-InstallExtensionControlledPrefIncognitoSessionOnly(Extension* extension,
-                                                   const std::string& key,
-                                                   base::Value* value) {
+    InstallExtensionControlledPrefIncognitoSessionOnly(Extension* extension,
+                                                       const std::string& key,
+                                                       base::Value value) {
   EnsureExtensionInstalled(extension);
-  test_preference_api_.SetExtensionControlledPref(
-      extension->id(), key, kExtensionPrefsScopeIncognitoSessionOnly, value);
+  prefs_helper_.SetExtensionControlledPref(
+      extension->id(), key, kExtensionPrefsScopeIncognitoSessionOnly,
+      std::move(value));
 }
 
 void ExtensionControlledPrefsTest::InstallExtension(Extension* extension) {
@@ -157,10 +130,8 @@ void ExtensionControlledPrefsTest::UninstallExtension(
 void ExtensionControlledPrefsTest::EnsureExtensionInstalled(
     Extension* extension) {
   // Install extension the first time a preference is set for it.
-  Extension* extensions[] = { extension1(),
-                              extension2(),
-                              extension3(),
-                              extension4() };
+  Extension* extensions[] = {extension1(), extension2(), extension3(),
+                             extension4(), internal_extension()};
   for (size_t i = 0; i < kNumInstalledExtensions; ++i) {
     if (extension == extensions[i] && !installed_[i]) {
       prefs()->OnExtensionInstalled(extension,
@@ -176,24 +147,22 @@ void ExtensionControlledPrefsTest::EnsureExtensionInstalled(
 
 void ExtensionControlledPrefsTest::EnsureExtensionUninstalled(
     const std::string& extension_id) {
-  Extension* extensions[] = { extension1(),
-                              extension2(),
-                              extension3(),
-                              extension4() };
+  Extension* extensions[] = {extension1(), extension2(), extension3(),
+                             extension4(), internal_extension()};
   for (size_t i = 0; i < kNumInstalledExtensions; ++i) {
     if (extensions[i]->id() == extension_id) {
       installed_[i] = false;
       break;
     }
   }
-  prefs()->OnExtensionUninstalled(extension_id, Manifest::INTERNAL, false);
+  prefs()->OnExtensionUninstalled(extension_id,
+                                  mojom::ManifestLocation::kInternal, false);
 }
 
 class ControlledPrefsInstallOneExtension
     : public ExtensionControlledPrefsTest {
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
+    InstallExtensionControlledPref(extension1(), kPref1, base::Value("val1"));
   }
   void Verify() override {
     std::string actual = prefs()->pref_service()->GetString(kPref1);
@@ -208,12 +177,11 @@ class ControlledPrefsInstallIncognitoPersistent
     : public ExtensionControlledPrefsTest {
  public:
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
+    InstallExtensionControlledPref(extension1(), kPref1, base::Value("val1"));
     InstallExtensionControlledPrefIncognito(extension1(), kPref1,
-                                            new base::Value("val2"));
-    std::unique_ptr<PrefService> incog_prefs(
-        prefs_.CreateIncognitoPrefService());
+                                            base::Value("val2"));
+    std::unique_ptr<PrefService> incog_prefs =
+        prefs_.CreateIncognitoPrefService();
     std::string actual = incog_prefs->GetString(kPref1);
     EXPECT_EQ("val2", actual);
   }
@@ -223,8 +191,8 @@ class ControlledPrefsInstallIncognitoPersistent
     std::string actual = prefs()->pref_service()->GetString(kPref1);
     EXPECT_EQ("val1", actual);
     // Incognito pref service shall see incognito values.
-    std::unique_ptr<PrefService> incog_prefs(
-        prefs_.CreateIncognitoPrefService());
+    std::unique_ptr<PrefService> incog_prefs =
+        prefs_.CreateIncognitoPrefService();
     actual = incog_prefs->GetString(kPref1);
     EXPECT_EQ("val2", actual);
   }
@@ -239,12 +207,11 @@ class ControlledPrefsInstallIncognitoSessionOnly
   ControlledPrefsInstallIncognitoSessionOnly() : iteration_(0) {}
 
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
+    InstallExtensionControlledPref(extension1(), kPref1, base::Value("val1"));
     InstallExtensionControlledPrefIncognitoSessionOnly(extension1(), kPref1,
-                                                       new base::Value("val2"));
-    std::unique_ptr<PrefService> incog_prefs(
-        prefs_.CreateIncognitoPrefService());
+                                                       base::Value("val2"));
+    std::unique_ptr<PrefService> incog_prefs =
+        prefs_.CreateIncognitoPrefService();
     std::string actual = incog_prefs->GetString(kPref1);
     EXPECT_EQ("val2", actual);
   }
@@ -255,8 +222,8 @@ class ControlledPrefsInstallIncognitoSessionOnly
     // Incognito pref service shall see session-only incognito values only
     // during first run. Once the pref service was reloaded, all values shall be
     // discarded.
-    std::unique_ptr<PrefService> incog_prefs(
-        prefs_.CreateIncognitoPrefService());
+    std::unique_ptr<PrefService> incog_prefs =
+        prefs_.CreateIncognitoPrefService();
     actual = incog_prefs->GetString(kPref1);
     if (iteration_ == 0) {
       EXPECT_EQ("val2", actual);
@@ -272,19 +239,14 @@ TEST_F(ControlledPrefsInstallIncognitoSessionOnly,
 
 class ControlledPrefsUninstallExtension : public ExtensionControlledPrefsTest {
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
-    InstallExtensionControlledPref(extension1(), kPref2,
-                                   new base::Value("val2"));
+    InstallExtensionControlledPref(extension1(), kPref1, base::Value("val1"));
+    InstallExtensionControlledPref(extension1(), kPref2, base::Value("val2"));
     scoped_refptr<ContentSettingsStore> store = content_settings_store();
     ContentSettingsPattern pattern =
         ContentSettingsPattern::FromString("http://[*.]example.com");
-    store->SetExtensionContentSetting(extension1()->id(),
-                                      pattern, pattern,
-                                      CONTENT_SETTINGS_TYPE_IMAGES,
-                                      std::string(),
-                                      CONTENT_SETTING_BLOCK,
-                                      kExtensionPrefsScopeRegular);
+    store->SetExtensionContentSetting(
+        extension1()->id(), pattern, pattern, ContentSettingsType::IMAGES,
+        CONTENT_SETTING_BLOCK, kExtensionPrefsScopeRegular);
 
     UninstallExtension(extension1()->id());
   }
@@ -314,8 +276,8 @@ class ControlledPrefsNotifyWhenNeeded : public ExtensionControlledPrefsTest {
     registrar.Add(kPref1, observer.GetCallback());
 
     MockPrefChangeCallback incognito_observer(prefs()->pref_service());
-    std::unique_ptr<PrefService> incog_prefs(
-        prefs_.CreateIncognitoPrefService());
+    std::unique_ptr<PrefService> incog_prefs =
+        prefs_.CreateIncognitoPrefService();
     PrefChangeRegistrar incognito_registrar;
     incognito_registrar.Init(incog_prefs.get());
     incognito_registrar.Add(kPref1, incognito_observer.GetCallback());
@@ -324,7 +286,7 @@ class ControlledPrefsNotifyWhenNeeded : public ExtensionControlledPrefsTest {
     EXPECT_CALL(observer, OnPreferenceChanged(_));
     EXPECT_CALL(incognito_observer, OnPreferenceChanged(_));
     InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("https://www.chromium.org"));
+                                   base::Value("https://www.chromium.org"));
     Mock::VerifyAndClearExpectations(&observer);
     Mock::VerifyAndClearExpectations(&incognito_observer);
 
@@ -332,7 +294,7 @@ class ControlledPrefsNotifyWhenNeeded : public ExtensionControlledPrefsTest {
     EXPECT_CALL(observer, OnPreferenceChanged(_)).Times(0);
     EXPECT_CALL(incognito_observer, OnPreferenceChanged(_)).Times(0);
     InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("https://www.chromium.org"));
+                                   base::Value("https://www.chromium.org"));
     Mock::VerifyAndClearExpectations(&observer);
     Mock::VerifyAndClearExpectations(&incognito_observer);
 
@@ -340,22 +302,22 @@ class ControlledPrefsNotifyWhenNeeded : public ExtensionControlledPrefsTest {
     EXPECT_CALL(observer, OnPreferenceChanged(_));
     EXPECT_CALL(incognito_observer, OnPreferenceChanged(_));
     InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("chrome://newtab"));
+                                   base::Value("chrome://newtab"));
     Mock::VerifyAndClearExpectations(&observer);
     Mock::VerifyAndClearExpectations(&incognito_observer);
     // Change only incognito persistent value.
     EXPECT_CALL(observer, OnPreferenceChanged(_)).Times(0);
     EXPECT_CALL(incognito_observer, OnPreferenceChanged(_));
-    InstallExtensionControlledPrefIncognito(
-        extension1(), kPref1, new base::Value("chrome://newtab2"));
+    InstallExtensionControlledPrefIncognito(extension1(), kPref1,
+                                            base::Value("chrome://newtab2"));
     Mock::VerifyAndClearExpectations(&observer);
     Mock::VerifyAndClearExpectations(&incognito_observer);
 
     // Change only incognito session-only value.
     EXPECT_CALL(observer, OnPreferenceChanged(_)).Times(0);
     EXPECT_CALL(incognito_observer, OnPreferenceChanged(_));
-    InstallExtensionControlledPrefIncognito(
-        extension1(), kPref1, new base::Value("chrome://newtab3"));
+    InstallExtensionControlledPrefIncognito(extension1(), kPref1,
+                                            base::Value("chrome://newtab3"));
     Mock::VerifyAndClearExpectations(&observer);
     Mock::VerifyAndClearExpectations(&incognito_observer);
 
@@ -380,12 +342,11 @@ TEST_F(ControlledPrefsNotifyWhenNeeded,
 // Tests disabling an extension.
 class ControlledPrefsDisableExtension : public ExtensionControlledPrefsTest {
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
+    InstallExtensionControlledPref(extension1(), kPref1, base::Value("val1"));
     std::string actual = prefs()->pref_service()->GetString(kPref1);
     EXPECT_EQ("val1", actual);
     prefs()->SetExtensionDisabled(extension1()->id(),
-                                  Extension::DISABLE_USER_ACTION);
+                                  disable_reason::DISABLE_USER_ACTION);
   }
   void Verify() override {
     std::string actual = prefs()->pref_service()->GetString(kPref1);
@@ -397,10 +358,9 @@ TEST_F(ControlledPrefsDisableExtension, ControlledPrefsDisableExtension) { }
 // Tests disabling and reenabling an extension.
 class ControlledPrefsReenableExtension : public ExtensionControlledPrefsTest {
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
+    InstallExtensionControlledPref(extension1(), kPref1, base::Value("val1"));
     prefs()->SetExtensionDisabled(extension1()->id(),
-                                  Extension::DISABLE_USER_ACTION);
+                                  disable_reason::DISABLE_USER_ACTION);
     prefs()->SetExtensionEnabled(extension1()->id());
   }
   void Verify() override {
@@ -414,16 +374,10 @@ class ControlledPrefsSetExtensionControlledPref
     : public ExtensionControlledPrefsTest {
  public:
   void Initialize() override {
-    base::Value* v1 = new base::Value("https://www.chromium.org");
-    base::Value* v2 = new base::Value("https://www.chromium.org");
-    base::Value* v1i = new base::Value("https://www.chromium.org");
-    base::Value* v2i = new base::Value("https://www.chromium.org");
-    // Ownership is taken, value shall not be deleted.
-    InstallExtensionControlledPref(extension1(), kPref1, v1);
-    InstallExtensionControlledPrefIncognito(extension1(), kPref1, v1i);
-    // Make sure there is no memory leak and both values are deleted.
-    InstallExtensionControlledPref(extension1(), kPref1, v2);
-    InstallExtensionControlledPrefIncognito(extension1(), kPref1, v2i);
+    InstallExtensionControlledPref(extension1(), kPref1,
+                                   base::Value("https://www.chromium.org"));
+    InstallExtensionControlledPrefIncognito(
+        extension1(), kPref1, base::Value("https://www.chromium.org"));
     prefs_.RecreateExtensionPrefs();
   }
 
@@ -440,19 +394,33 @@ class ControlledPrefsDisableExtensions : public ExtensionControlledPrefsTest {
       : iteration_(0) {}
   ~ControlledPrefsDisableExtensions() override {}
   void Initialize() override {
-    InstallExtensionControlledPref(extension1(), kPref1,
-                                   new base::Value("val1"));
+    InstallExtensionControlledPref(internal_extension(), kPref1,
+                                   base::Value("internal extension value"));
+
+    EXPECT_TRUE(Manifest::IsExternalLocation(extension1()->location()));
+    InstallExtensionControlledPref(extension1(), kPref2,
+                                   base::Value("external extension value"));
     // This becomes only active in the second verification phase.
     prefs_.set_extensions_disabled(true);
   }
   void Verify() override {
-    std::string actual = prefs()->pref_service()->GetString(kPref1);
+    // Internal extensions are not loaded with --disable-extensions. This means
+    // that the preference will be reset on the second verification run (when
+    // the ExtensionPrefs are recreated).
+    std::string pref1_actual = prefs()->pref_service()->GetString(kPref1);
     if (iteration_ == 0) {
-      EXPECT_EQ("val1", actual);
+      EXPECT_EQ("internal extension value", pref1_actual);
       ++iteration_;
     } else {
-      EXPECT_EQ(kDefaultPref1, actual);
+      EXPECT_EQ(kDefaultPref1, pref1_actual);
     }
+
+    // External extensions are loaded even when extensions are disabled (though
+    // they likely shouldn't be, see https://crbug.com/833540). Because of this,
+    // the preference should still be controlled by the external extension.
+    // Regression test for https://crbug.com/828295.
+    std::string pref2_actual = prefs()->pref_service()->GetString(kPref2);
+    EXPECT_EQ("external extension value", pref2_actual);
   }
 
  private:
