@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,35 +8,38 @@
 #include <memory>
 #include <vector>
 
+#include "base/feature_list.h"
 #include "base/files/file_enumerator.h"
 #include "base/files/file_path.h"
-#include "base/macros.h"
 #include "base/no_destructor.h"
 #include "base/path_service.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/task_environment.h"
+#include "build/build_config.h"
 #include "components/autofill/core/browser/autofill_test_utils.h"
 #include "components/autofill/core/browser/autofill_type.h"
-#include "components/autofill/core/browser/country_names.h"
-#include "components/autofill/core/browser/data_driven_test.h"
+#include "components/autofill/core/browser/data_model/autofill_profile_comparator.h"
 #include "components/autofill/core/browser/form_data_importer.h"
 #include "components/autofill/core/browser/form_structure.h"
+#include "components/autofill/core/browser/geo/country_names.h"
 #include "components/autofill/core/browser/personal_data_manager.h"
 #include "components/autofill/core/browser/test_autofill_client.h"
+#include "components/autofill/core/common/autofill_features.h"
 #include "components/autofill/core/common/form_data.h"
+#include "testing/data_driven_testing/data_driven_test.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
 
-#if defined(OS_MACOSX)
+#if BUILDFLAG(IS_APPLE)
 #include "base/mac/foundation_util.h"
 #endif
 
 namespace autofill {
 
 namespace {
-
+const base::FilePath::CharType kFeatureName[] = FILE_PATH_LITERAL("autofill");
 const base::FilePath::CharType kTestName[] = FILE_PATH_LITERAL("merge");
 const base::FilePath::CharType kFileNamePattern[] = FILE_PATH_LITERAL("*.in");
 
@@ -69,7 +72,8 @@ const base::FilePath& GetTestDataDir() {
 
 const std::vector<base::FilePath> GetTestFiles() {
   base::FilePath dir = GetTestDataDir();
-  dir = dir.AppendASCII("autofill").AppendASCII("merge").AppendASCII("input");
+
+  dir = dir.AppendASCII("autofill").Append(kTestName).AppendASCII("input");
   base::FileEnumerator input_files(dir, false, base::FileEnumerator::FILES,
                                    kFileNamePattern);
   std::vector<base::FilePath> files;
@@ -79,9 +83,9 @@ const std::vector<base::FilePath> GetTestFiles() {
   }
   std::sort(files.begin(), files.end());
 
-#if defined(OS_MACOSX)
+#if BUILDFLAG(IS_APPLE)
   base::mac::ClearAmIBundledCache();
-#endif  // defined(OS_MACOSX)
+#endif  // BUILDFLAG(IS_APPLE)
 
   return files;
 }
@@ -89,16 +93,15 @@ const std::vector<base::FilePath> GetTestFiles() {
 // Serializes the |profiles| into a string.
 std::string SerializeProfiles(const std::vector<AutofillProfile*>& profiles) {
   std::string result;
-  for (size_t i = 0; i < profiles.size(); ++i) {
+  for (auto* profile : profiles) {
     result += kProfileSeparator;
     result += "\n";
     for (const ServerFieldType& type : kProfileFieldTypes) {
-      base::string16 value = profiles[i]->GetRawInfo(type);
-      result += AutofillType(type).ToString();
+      std::u16string value = profile->GetRawInfo(type);
+      result += AutofillType::ServerFieldTypeToString(type);
       result += kFieldSeparator;
       if (!value.empty()) {
-        base::ReplaceFirstSubstringAfterOffset(
-            &value, 0, base::ASCIIToUTF16("\\n"), base::ASCIIToUTF16("\n"));
+        base::ReplaceFirstSubstringAfterOffset(&value, 0, u"\\n", u"\n");
         result += " ";
         result += base::UTF16ToUTF8(value);
       }
@@ -112,6 +115,10 @@ std::string SerializeProfiles(const std::vector<AutofillProfile*>& profiles) {
 class PersonalDataManagerMock : public PersonalDataManager {
  public:
   PersonalDataManagerMock();
+
+  PersonalDataManagerMock(const PersonalDataManagerMock&) = delete;
+  PersonalDataManagerMock& operator=(const PersonalDataManagerMock&) = delete;
+
   ~PersonalDataManagerMock() override;
 
   // Reset the saved profiles.
@@ -123,16 +130,12 @@ class PersonalDataManagerMock : public PersonalDataManager {
 
  private:
   std::vector<std::unique_ptr<AutofillProfile>> profiles_;
-
-  DISALLOW_COPY_AND_ASSIGN(PersonalDataManagerMock);
 };
 
 PersonalDataManagerMock::PersonalDataManagerMock()
-    : PersonalDataManager("en-US") {
-}
+    : PersonalDataManager("en-US", "US") {}
 
-PersonalDataManagerMock::~PersonalDataManagerMock() {
-}
+PersonalDataManagerMock::~PersonalDataManagerMock() = default;
 
 void PersonalDataManagerMock::Reset() {
   profiles_.clear();
@@ -140,11 +143,14 @@ void PersonalDataManagerMock::Reset() {
 
 std::string PersonalDataManagerMock::SaveImportedProfile(
     const AutofillProfile& profile) {
-  std::vector<AutofillProfile> profiles;
-  std::string merged_guid =
-      MergeProfile(profile, &profiles_, "en-US", &profiles);
-  if (merged_guid == profile.guid())
-    profiles_.push_back(std::make_unique<AutofillProfile>(profile));
+  std::vector<AutofillProfile> new_profiles;
+  std::string merged_guid = AutofillProfileComparator::MergeProfile(
+      profile, profiles_, "en-US", &new_profiles);
+
+  profiles_.clear();
+  for (const AutofillProfile& it : new_profiles) {
+    profiles_.push_back(std::make_unique<AutofillProfile>(it));
+  }
   return merged_guid;
 }
 
@@ -162,8 +168,12 @@ std::vector<AutofillProfile*> PersonalDataManagerMock::GetProfiles() const {
 // corresponding output file is a dump of the saved profiles that result from
 // importing the input profiles. The output file format is identical to the
 // input format.
-class AutofillMergeTest : public DataDrivenTest,
+class AutofillMergeTest : public testing::DataDrivenTest,
                           public testing::TestWithParam<base::FilePath> {
+ public:
+  AutofillMergeTest(const AutofillMergeTest&) = delete;
+  AutofillMergeTest& operator=(const AutofillMergeTest&) = delete;
+
  protected:
   AutofillMergeTest();
   ~AutofillMergeTest() override;
@@ -183,30 +193,32 @@ class AutofillMergeTest : public DataDrivenTest,
   // Deserializes |str| into a field type.
   ServerFieldType StringToFieldType(const std::string& str);
 
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   TestAutofillClient autofill_client_;
   PersonalDataManagerMock personal_data_;
   std::unique_ptr<FormDataImporter> form_data_importer_;
 
  private:
   std::map<std::string, ServerFieldType> string_to_field_type_map_;
-
-  DISALLOW_COPY_AND_ASSIGN(AutofillMergeTest);
 };
 
-AutofillMergeTest::AutofillMergeTest() : DataDrivenTest(GetTestDataDir()) {
+AutofillMergeTest::AutofillMergeTest()
+    : testing::DataDrivenTest(GetTestDataDir(), kFeatureName, kTestName) {
   CountryNames::SetLocaleString("en-US");
   for (size_t i = NO_SERVER_DATA; i < MAX_VALID_FIELD_TYPE; ++i) {
-    ServerFieldType field_type = static_cast<ServerFieldType>(i);
-    string_to_field_type_map_[AutofillType(field_type).ToString()] = field_type;
+    ServerFieldType field_type = ToSafeServerFieldType(i, MAX_VALID_FIELD_TYPE);
+    if (field_type == MAX_VALID_FIELD_TYPE)
+      continue;
+    string_to_field_type_map_[AutofillType::ServerFieldTypeToString(
+        field_type)] = field_type;
   }
 }
 
-AutofillMergeTest::~AutofillMergeTest() {
-}
+AutofillMergeTest::~AutofillMergeTest() = default;
 
 void AutofillMergeTest::SetUp() {
   test::DisableSystemServices(nullptr);
+  personal_data_.set_auto_accept_address_imports_for_testing(true);
   form_data_importer_ = std::make_unique<FormDataImporter>(
       &autofill_client_,
       /*payments::PaymentsClient=*/nullptr, &personal_data_, "en");
@@ -228,8 +240,8 @@ void AutofillMergeTest::MergeProfiles(const std::string& profiles,
 
   // Create a test form.
   FormData form;
-  form.name = base::ASCIIToUTF16("MyTestForm");
-  form.origin = GURL("https://www.example.com/origin.html");
+  form.name = u"MyTestForm";
+  form.url = GURL("https://www.example.com/origin.html");
   form.action = GURL("https://www.example.com/action.html");
 
   // Parse the input line by line.
@@ -242,15 +254,13 @@ void AutofillMergeTest::MergeProfiles(const std::string& profiles,
       size_t separator_pos = line.find(kFieldSeparator);
       ASSERT_NE(std::string::npos, separator_pos)
           << "Wrong format for separator on line " << i;
-      base::string16 field_type =
+      std::u16string field_type =
           base::UTF8ToUTF16(line.substr(0, separator_pos));
       do {
         ++separator_pos;
       } while (separator_pos < line.size() && line[separator_pos] == ' ');
-      base::string16 value =
-          base::UTF8ToUTF16(line.substr(separator_pos));
-      base::ReplaceFirstSubstringAfterOffset(
-          &value, 0, base::ASCIIToUTF16("\\n"), base::ASCIIToUTF16("\n"));
+      std::u16string value = base::UTF8ToUTF16(line.substr(separator_pos));
+      base::ReplaceFirstSubstringAfterOffset(&value, 0, u"\\n", u"\n");
 
       FormFieldData field;
       field.label = field_type;
@@ -266,25 +276,26 @@ void AutofillMergeTest::MergeProfiles(const std::string& profiles,
     if ((i > 0 && line == kProfileSeparator) || i == lines.size() - 1) {
       // Reached the end of a profile.  Try to import it.
       FormStructure form_structure(form);
-      for (size_t i = 0; i < form_structure.field_count(); ++i) {
+      for (size_t j = 0; j < form_structure.field_count(); ++j) {
         // Set the heuristic type for each field, which is currently serialized
         // into the field's name.
         AutofillField* field =
-            const_cast<AutofillField*>(form_structure.field(i));
+            const_cast<AutofillField*>(form_structure.field(j));
         ServerFieldType type =
             StringToFieldType(base::UTF16ToUTF8(field->name));
-        field->set_heuristic_type(type);
+        field->set_heuristic_type(GetActivePatternSource(), type);
       }
-      form_structure.IdentifySections(false);
 
-      // Import the profile.
-      std::unique_ptr<CreditCard> imported_credit_card;
-      form_data_importer_->ImportFormData(form_structure,
-                                          true,  // address autofill enabled,
-                                          true,  // credit card autofill enabled
-                                          false,  // should return local card
-                                          &imported_credit_card);
-      EXPECT_FALSE(imported_credit_card);
+      // Extract the profile.
+      auto extracted_data = form_data_importer_->ExtractFormData(
+          form_structure,
+          /*profile_autofill_enabled=*/true,
+          /*payment_methods_autofill_enabled=*/true);
+      form_data_importer_->ProcessAddressProfileImportCandidates(
+          extracted_data.address_profile_import_candidates,
+          /*allow_prompt=*/true);
+      EXPECT_FALSE(extracted_data.credit_card_import_candidate);
+      EXPECT_FALSE(extracted_data.extracted_upi_id.has_value());
 
       // Clear the |form| to start a new profile.
       form.fields.clear();
@@ -300,10 +311,11 @@ ServerFieldType AutofillMergeTest::StringToFieldType(const std::string& str) {
 
 TEST_P(AutofillMergeTest, DataDrivenMergeProfiles) {
   const bool kIsExpectedToPass = true;
-  RunOneDataDrivenTest(GetParam(), GetOutputDirectory(kTestName),
-                       kIsExpectedToPass);
+  RunOneDataDrivenTest(GetParam(), GetOutputDirectory(), kIsExpectedToPass);
 }
 
-INSTANTIATE_TEST_CASE_P(, AutofillMergeTest, testing::ValuesIn(GetTestFiles()));
+INSTANTIATE_TEST_SUITE_P(All,
+                         AutofillMergeTest,
+                         testing::ValuesIn(GetTestFiles()));
 
 }  // namespace autofill
